@@ -460,17 +460,10 @@ inline Rcpp::RObject imageDataToArray (const nifti_image *source)
     }
 }
 
-inline void finaliseNiftiImage (SEXP xptr)
-{
-    NiftiImage *object = (NiftiImage *) R_ExternalPtrAddr(xptr);
-    object->setPersistence(false);
-    delete object;
-    R_ClearExternalPtr(xptr);
-}
-
-inline void addAttributes (Rcpp::RObject &object, nifti_image *source, const bool realDim = true)
+inline void addAttributes (const SEXP pointer, const NiftiImage &source, const bool realDim = true, const bool includeXptr = true)
 {
     const int nDims = source->dim[0];
+    Rcpp::RObject object(pointer);
     Rcpp::IntegerVector dim(source->dim+1, source->dim+1+nDims);
 
     if (realDim)
@@ -493,11 +486,11 @@ inline void addAttributes (Rcpp::RObject &object, nifti_image *source, const boo
         object.attr("pixunits") = pixunits;
     }
     
-    NiftiImage *wrappedSource = new NiftiImage(source, true);
-    wrappedSource->setPersistence(true);
-    Rcpp::XPtr<NiftiImage> xptr(wrappedSource);
-    R_RegisterCFinalizerEx(SEXP(xptr), &finaliseNiftiImage, FALSE);
-    object.attr(".nifti_image_ptr") = xptr;
+    if (includeXptr)
+    {
+        Rcpp::XPtr<NiftiImage> xptr(new NiftiImage(source,false));
+        object.attr(".nifti_image_ptr") = xptr;
+    }
 }
 
 #endif // USING_R
